@@ -436,12 +436,12 @@ typedef struct
     int mPid;
     int mStatus;
     int mPGroup;
-} sOtherCProg;
+} sStrayCProg;
 
-extern vector_obj* gOtherCProgs;
+extern vector_obj* gStrayCProgs;
 
-sOtherCProg* sOtherCprog_new(int pid, int status, int pgroup);
-void sOtherCProg_delete(sOtherCProg* self);
+sStrayCProg* sStrayCprog_new(int pid, int status, int pgroup);
+void sStrayCProg_delete(sStrayCProg* self);
 
 typedef struct {
     pid_t mPid;
