@@ -503,6 +503,12 @@ void saphire_set_local_var(char* name, char* value)
     }
 }
 
+void saphire_set_global_var(char* name, char* value)
+{
+    saphire_delete_same_name_var(name);
+    hash_put(gGlobals, name, STRING_NEW(value));
+}
+
 void saphire_delete_same_name_var(char* name)
 {
     string_obj* global = hash_item(gGlobals, name);
@@ -542,12 +548,6 @@ void saphire_delete_same_name_var(char* name)
     }
 }
 
-void saphire_set_global_var(char* name, char* value)
-{
-    saphire_delete_same_name_var(name);
-    hash_put(gGlobals, name, STRING_NEW(value));
-}
-
 char* saphire_get_global_var(char* name)
 {
     string_obj* global = hash_item(gGlobals, name);
@@ -609,6 +609,16 @@ void saphire_delete_local_var(char* name)
             string_delete(var);
             hash_erase(gGlobals, name);
         }
+
+        char name_size[1024];
+        snprintf(name_size, 1024, "%s_size", name);
+
+        string_obj* var_size = hash_item(gGlobals, name_size);
+
+        if(var_size) {
+            string_delete(var_size);
+            hash_erase(gGlobals, name_size);
+        }
     }
     else {
         hash_obj* top_stack 
@@ -620,6 +630,16 @@ void saphire_delete_local_var(char* name)
             string_delete(var);
             hash_erase(top_stack, name);
         }
+
+        char name_size[1024];
+        snprintf(name_size, 1024, "%s_size", name);
+
+        string_obj* var_size = hash_item(gGlobals, name_size);
+
+        if(var_size) {
+            string_delete(var_size);
+            hash_erase(gGlobals, name_size);
+        }
     }
 }
 
@@ -627,6 +647,8 @@ void saphire_delete_local_var(char* name)
 // シグナル処理
 //////////////////////////////////////////////////////////
 volatile BOOL gKitutukiSigInt = FALSE;
+volatile BOOL gKitutukiSigTstp = FALSE;
+volatile BOOL gKitutukiSigCont = FALSE;
 volatile BOOL gSigUser = FALSE;
 
 void sig_int()
@@ -637,6 +659,11 @@ void sig_int()
 fprintf(stderr, "SIGINT!!\n");
 #endif
 }
+void sig_int_optc()
+{
+    gKitutukiSigInt = TRUE; 
+fprintf(stderr, "SIGINT!!\n");
+}
 void sig_tstp()
 {
     gKitutukiSigInt = TRUE; 
@@ -646,6 +673,21 @@ fprintf(stderr, "SIGTSTP!!\n");
 #endif
 }
 
+void sig_tstp_optc()
+{
+    gKitutukiSigTstp = TRUE;
+    //gKitutukiSigInt = TRUE; 
+    kill(getpid(), SIGSTOP);
+    //killpg(getpgid(), SIGTERM);
+//fprintf(stderr, "SIGTSTP!!\n");
+}
+
+void sig_cont_optc()
+{
+    gKitutukiSigCont = TRUE;
+    killpg(getpgid(), SIGCONT);
+}
+
 void sig_user()
 {
     gSigUser = TRUE;
@@ -824,6 +866,90 @@ void saphire_set_signal()
     if(saphire_set_signal_other) saphire_set_signal_other();
 }
 
+void saphire_set_signal_optc()
+{
+    struct sigaction sa;
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_sigaction = sigchld_action;
+    sa.sa_flags = SA_SIGINFO;
+    if(sigaction(SIGCHLD, &sa, NULL) < 0) {
+        perror("sigaction1");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_flags = SA_SIGINFO;
+    sa.sa_handler = SIG_IGN; //sig_int_optc;
+    if(sigaction(SIGINT, &sa, NULL) < 0) {
+        perror("sigaction2");
+        exit(1);
+    }
+    sa.sa_flags = SA_SIGINFO;
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = sig_cont_optc;
+    if(sigaction(SIGCONT, &sa, NULL) < 0) {
+        perror("sigaction3");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = SIG_IGN;
+    if(sigaction(SIGWINCH, &sa, NULL) < 0) {
+        perror("sigaction4");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = SIG_IGN;
+    sa.sa_flags = 0;
+    if(sigaction(SIGTTOU, &sa, NULL) < 0) {
+        perror("sigaction5");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = SIG_IGN;
+    sa.sa_flags = 0;
+    if(sigaction(SIGTTIN, &sa, NULL) < 0) {
+        perror("sigaction6");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = sig_tstp_optc;
+    sa.sa_flags = 0;
+    if(sigaction(SIGTSTP, &sa, NULL) < 0) {
+        perror("sigaction7");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = SIG_IGN;
+    sa.sa_flags = 0;
+    if(sigaction(SIGQUIT, &sa, NULL) < 0) {
+        perror("sigaction8");
+        exit(1);
+    }
+
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = sig_user;
+    sa.sa_flags = 0;
+    if(sigaction(SIGUSR1, &sa, NULL) < 0) {
+        perror("sigaction9");
+        exit(1);
+    }
+    memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = SIG_IGN;
+    sa.sa_flags = 0;
+    if(sigaction(SIGPIPE, &sa, NULL) < 0) {
+        perror("sigaction10");
+        exit(1);
+    }
+
+    if(saphire_set_signal_other) saphire_set_signal_other();
+}
+
 //////////////////////////////////////////////////////////
 // ジョブ
 //////////////////////////////////////////////////////////
@@ -2208,11 +2334,11 @@ void sFunction_view(sFunction* self)
 /////////////////////////////////////////////////////////
 // run_wait_cprog で使う
 /////////////////////////////////////////////////////////
-vector_obj* gOtherCProgs;
+vector_obj* gStrayCProgs;
 
-sOtherCProg* sOtherCprog_new(int pid, int status, int pgroup)
+sStrayCProg* sStrayCprog_new(int pid, int status, int pgroup)
 {
-    sOtherCProg* self = MALLOC(sizeof(sOtherCProg));
+    sStrayCProg* self = MALLOC(sizeof(sStrayCProg));
     
     self->mPid = pid;
     self->mStatus = status;
@@ -2221,7 +2347,7 @@ sOtherCProg* sOtherCprog_new(int pid, int status, int pgroup)
     return self;
 }
 
-void sOtherCProg_delete(sOtherCProg* self)
+void sStrayCProg_delete(sStrayCProg* self)
 {
     FREE(self);
 }
@@ -2543,7 +2669,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gFileHandles = HASH_NEW(100);
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "1.0.6", 1);
+    setenv("SAPHIRE_VERSION", "1.0.7", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2569,7 +2695,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gFReadBuffers = HASH_NEW(100);
     gFWriteBuffers = HASH_NEW(512);
     
-    gOtherCProgs = VECTOR_NEW(10);
+    gStrayCProgs = VECTOR_NEW(10);
 
     gStackFrame = VECTOR_NEW(5);
     //vector_add(gStackFrame, HASH_NEW(30));
@@ -2609,7 +2735,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     else 
         strcpy(gTmpDir, "/tmp");
 
-    gMatchRestart = HASH_NEW(100);
+    gMatchRestart = HASH_NEW(1000);
     gStackTraceFName = VECTOR_NEW(30);
     gStackTraceLineNum = VECTOR_NEW(30);
 
@@ -2770,10 +2896,10 @@ void saphire_final()
     }
     hash_delete(gFWriteBuffers);
     
-    for(i=0; i<vector_size(gOtherCProgs); i++) {
-        sOtherCProg_delete(vector_item(gOtherCProgs, i));
+    for(i=0; i<vector_size(gStrayCProgs); i++) {
+        sStrayCProg_delete(vector_item(gStrayCProgs, i));
     }
-    vector_delete(gOtherCProgs);
+    vector_delete(gStrayCProgs);
 
     if(msgctl(gMsgId, IPC_RMID, 0) == -1) {
         fprintf(stderr, "msgctl(IPC_RMID) failed\n");
